/*
 nRF24 RC Car RX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 2/11/2013
 Last Modified: 10/11/2013

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define SERVO_RIGHT 250
#define SERVO_CENTER 500
#define SERVO_LEFT 750

#define LEFT_RIGHT_CTRL 0
#define FORWARD_REVERSE_CTRL 1

#define MOTOR_FORWARD 1
#define MOTOR_REVERSE 0

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// Write data using SPI
void spi_write_data(uint8_t * dataout, uint8_t len) {
	uint8_t i;
	for (i = 0; i < len; i++) {
		spi_transfer(dataout[i]);
	}
}

// Read data using SPI
void spi_read_data(uint8_t * datain, uint8_t len) {
	uint8_t i;
	for (i = 0; i < len; i++) {
		datain[i] = spi_transfer(0x00);
	}
}

// Initialise the SPI
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	DDRA &= ~(1<<PA6); // SPI DI
	PORTA |= (1<<PA6); // SPI DI
}

// Turn off watchdog (because wdt_disable doesn't work?)
void wdt_disable_reset(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
}

// Setup the ATtiny
void setup(void) {
	// Disable the watchdog reset
	wdt_disable_reset(); 
	
	// Change 1 MHz to 8 MHz by changing clock prescaler to 1
	CLKPR = (1<<CLKPCE); // Prescaler enable
	CLKPR = 0; // Clock division factor 1 (0000)
	
	// Blink LED to show we are turning on
	DDRB |= (1<<PB1);
	for (int x = 0; x < 2; x++) {
		PORTB |= (1<<PB1);
		_delay_ms(500);
		PORTB &= ~(1<<PB1);
		_delay_ms(500);
	}
	PORTB |= (1<<PB1);
	
	// Enable watchdog reset for 4 seconds
	wdt_enable(WDTO_4S);
	
	spi_init(); // Initialise SPI
	
	// Set up servo
	DDRA |= (1<<PA0); // Servo connected to PA0
	TCNT1 = 0;
	TCCR1B = (1<<WGM13) | (1<<CS11); // PWM, Phase & Freq. Correct, 8 prescaler
	ICR1 = 10000; // Sets frequency to 50Hz / 20ms
	OCR1A = SERVO_CENTER; // Sets 1ms (servo centre)
	sbi(TIMSK1, OCIE1A); // Output Compare A Match Interrupt Enable
	
	// Set up motor
	DDRA |= (1<<PA1); // Motor forward control (low N mosfet)
	DDRA |= (1<<PA2); // Motor reverse control (low N mosfet)
	
	sei(); // Turn on interrupts
}
