/*
 nRF24 RC Car TX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 2/11/2013
 Last Modified: 10/11/2013
 
 Transmit the ADC value on PA0/PA1 pins to the receiver.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 ADC0 - Left/Right
//         PB1  3|    |12  PA1 ADC1 - Forward/Reverse
//   Reset PB3  4|    |11  PA2
//      CE PB2  5|    |10  PA3 
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+

#define F_CPU 8000000 // 8 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"
#include "mirf.c"

#define leftrightPin 0
#define forwardreversePin 1
uint8_t data_out[2];

int main(void) {
	setup();
	mirf_init();
	_delay_ms(50);
	mirf_config();
	
	// Power up the nRF24L01
	TX_POWERUP;
	_delay_ms(3);
	
	while(1) {
		// Read the voltage of the pots
		data_out[0] = analog_read_ADCH(leftrightPin);
		data_out[1] = analog_read_ADCH(forwardreversePin);
		
		mirf_transmit_data();
	}
}

ISR(ADC_vect) { }
